package com.usky.demo.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.usky.demo.domain.*;
import com.usky.demo.mapper.QueryInfluxdbDataMapper;
import com.usky.demo.service.QueryInfluxdbDataService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.demo.service.rocketmq.MyProducer;
import com.usky.demo.service.utils.TsdbUtils;
import org.influxdb.dto.QueryResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2024-07-29
 */
@Service
public class QueryInfluxdbDataServiceImpl extends AbstractCrudService<QueryInfluxdbDataMapper, QueryInfluxdbData> implements QueryInfluxdbDataService {
    @Autowired
    private TsdbUtils tsdbUtils;

    @Resource
    private MyProducer myProducer;

    @Override
    public Map<String,Object> sendDeviceData(DeviceDataWriteVO writeVO){
        Map<String,Object> rec_map = new HashMap<>();
        DeviceDataInfoVO dataInfo = new DeviceDataInfoVO();
        Map<String,Object> metrics = writeVO.getMetrics();
        Map<String,String> tags = writeVO.getTags();
        if(metrics.size() > 0){
            Map<String,Object> mp = new HashMap<>();
            for(Map.Entry<String,Object> map:metrics.entrySet()){
                mp.put(map.getKey(),map.getValue());
            }
            dataInfo.setMetrics(mp);
            if(tags != null && tags.size() > 0){
                mp.clear();
                for(Map.Entry<String,String> map:tags.entrySet()){
                    mp.put(map.getKey(),map.getValue());
                }
                dataInfo.setTags(mp);
            }else{
                dataInfo.setTags(new HashMap<>());
            }
            dataInfo.setProduct_code(writeVO.getProductCode());
            dataInfo.setDevice_id(writeVO.getDeviceId());
            dataInfo.setTimestamp(writeVO.getTimestamp());

            myProducer.sendMessage("data-collector", JSONArray.toJSON(dataInfo).toString());
        }

        rec_map.put("code",200);
        rec_map.put("message","操作成功!");
        return rec_map;
    }

    @Override
    public LastResultVO queryLastDeviceData(String productCode, String deviceId){

        String tableName = productCode + "_" + deviceId;
        String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
        List<Map<String, Object>> metrics = tsdbUtils.fetchRecords(query);
        LastResultVO resultVO = new LastResultVO(deviceId,metrics);

        return resultVO;
    }

    @Override
    public List<LastResultVO> queryLastDeviceData(LastRequestVO requestVO){
        List<LastResultVO> result = new ArrayList<>();
        String productCode = requestVO.getProductCode();
        List<String> deviceIds = requestVO.getDeviceId();
        if(CollectionUtils.isNotEmpty(deviceIds)){
            for (int i = 0; i < deviceIds.size(); i++) {
                String tableName = productCode + "_" + deviceIds.get(i);
                String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
                List<Map<String, Object>> metrics = tsdbUtils.fetchRecords(query);
                LastResultVO resultVO = new LastResultVO(deviceIds.get(i),metrics);
                result.add(resultVO);
            }
        }

        return result;

    }

    @Override
    public HistoryResultVO queryHistoryDeviceData(String productCode,String deviceId,String startTime,String endTime){
        List<MetricVO> metricList = new ArrayList<>();
        String tableName = productCode + "_" + deviceId;
        String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
        QueryResult queryResult = tsdbUtils.query(query);
        if(queryResult.getResults().get(0).getSeries() != null){
            List<String> fields = queryResult.getResults().get(0).getSeries().get(0).getColumns();
            if(CollectionUtils.isNotEmpty(fields)){
                String fieldQuery = "SELECT *::field FROM \""+tableName+"\" where time >= '"+startTime+"' and time <= '"+endTime+"' tz('Asia/Shanghai')";
                List<Map<String,Object>> metircs = tsdbUtils.fetchRecords(fieldQuery);

                if(CollectionUtils.isNotEmpty(metircs)){
                    for (int j = 0; j < fields.size(); j++) {
                        List<Map<String,Object>> metircItems = new ArrayList<>();
                        if(!"time".equals(fields.get(j))){
                            for (int k = 0; k < metircs.size(); k++) {
                                String field = fields.get(j);
                                Map<String,Object> map = new HashMap<>();
                                map.put("timestamp",metircs.get(k).get("time"));
                                map.put("value",metircs.get(k).get(field));
                                metircItems.add(map);
                            }
                            MetricVO metricVO = new MetricVO(fields.get(j),metircItems);
                            metricList.add(metricVO);
                        }

                    }
                }

            }
        }

        HistoryResultVO resultVO = new HistoryResultVO(deviceId,metricList);
        return resultVO;
    }

    @Override
    public List<HistoryResultVO> queryHistoryDeviceData(HistoryRequestVO requestVO){
        List<HistoryResultVO> list = new ArrayList<>();
        String productCode = requestVO.getProductCode();
        List<String> deviceIds = requestVO.getDeviceId();
        String startTime = requestVO.getStartTime();
        String endTime = requestVO.getEndTime();
        for (int i = 0; i < deviceIds.size(); i++) {
            List<MetricVO> metricList = new ArrayList<>();
            String tableName = productCode + "_" + deviceIds.get(i);
            String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
            QueryResult queryResult = tsdbUtils.query(query);
            if(queryResult.getResults().get(0).getSeries() != null){
                List<String> fields = queryResult.getResults().get(0).getSeries().get(0).getColumns();
                if(CollectionUtils.isNotEmpty(fields)){
                    String fieldQuery = "SELECT *::field FROM \""+tableName+"\" where time >= '"+startTime+"' and time <= '"+endTime+"' tz('Asia/Shanghai')";
                    List<Map<String,Object>> metircs = tsdbUtils.fetchRecords(fieldQuery);

                    if(CollectionUtils.isNotEmpty(metircs)){
                        for (int j = 0; j < fields.size(); j++) {
                            List<Map<String,Object>> metircItems = new ArrayList<>();
                            if(!"time".equals(fields.get(j))){
                                for (int k = 0; k < metircs.size(); k++) {
                                    String field = fields.get(j);
                                    Map<String,Object> map = new HashMap<>();
                                    map.put("timestamp",metircs.get(k).get("time"));
                                    map.put("value",metircs.get(k).get(field));
                                    metircItems.add(map);
                                }
                                MetricVO metricVO = new MetricVO(fields.get(j),metircItems);
                                metricList.add(metricVO);
                            }

                        }
                    }

                }
            }

            HistoryResultVO resultVO = new HistoryResultVO(deviceIds.get(i),metricList);
            list.add(resultVO);
        }

        return list;
    }

}
